#include "framecpp/config.h"

#include <sstream>
#include <stdexcept>

#include "framecpp/Common/Description.hh"
#include "framecpp/Common/FrameStream.hh"
#include "framecpp/Common/FrSH.hh"

namespace FrameCPP
{
  namespace Common
  {
    Description::
    ~Description( )
    {
      for( description_container::const_iterator
	     cur = m_desc.begin( ),
	     last = m_desc.end( );
	   cur != last;
	   ++cur )
      {
	delete *cur;
      }
      m_desc.erase( m_desc.begin( ),
		    m_desc.end( ) );
    }

    const std::string& Description::
    GetName( ) const
    {
      static const std::string unknown( "UNKNOWN" );
      const FrSH* frsh = dynamic_cast< const FrSH* >( m_desc.front( ) );
      if ( frsh )
      {
	return frsh->name( );
      }
      
      return unknown;
    }

    void Description::
    Write( OFrameStream& Stream ) const
    {
      for ( description_container::const_iterator
	      cur = m_desc.begin( ),
	      last = m_desc.end( );
	    cur != last;
	    ++cur )
      {
	if (*cur)
	{
	  Stream.Write( (*cur) );
	}
      }
    }

    void Description::
    operator( )( const FrameSpec::Object& Obj )
    {
      if ( ( m_desc.size( ) == 0 )
	   && ( Obj.GetClass( ) != FrameSpec::Info::FSI_FR_SH ) )
      {
	std::ostringstream	msg;

	msg << "Expected object of class FrSH instead of: "
	    << Obj.GetClass( )
	  ;
	throw( std::logic_error( msg.str( ) ) );
      } else if ( ( m_desc.size( ) > 0 )
		  && ( Obj.GetClass( ) != FrameSpec::Info::FSI_FR_SE ) )
      {
	std::ostringstream	msg;

	msg << "Expected object of class FrSE instead of: "
	    << Obj.GetClass( )
	  ;
	throw( std::logic_error( msg.str( ) ) );
      }

      m_desc.push_back( Obj.Clone( ) );
    }

  } // namespace - Common
} // namespace - FrameCPP
