#ifndef FrameCPP_VERSION_6_FrTable_HH
#define FrameCPP_VERSION_6_FrTable_HH

#include <vector>

#include "framecpp/Version6/FrameSpec.hh"

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version4/FrTable.hh"

#include "framecpp/Version6/PTR_STRUCT.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/GPSTime.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    //
    //===================================================================
    using Previous::FrTableNPS;

    //===================================================================
    //
    //===================================================================
    typedef Version_4::
    FrTablePST< PTR_STRUCT,
		Common::SearchContainer< FrVect, &FrVect::GetName > >
    FrTablePS;

    //===================================================================
    /// \brief Implementation of FrTable according to the frame specification.
    //===================================================================
    class FrTable
      : public Common::FrameSpec::Object,
	public FrTableNPS,
	public FrTablePS
    {
    public:
      //: Default constructor
      FrTable( );
      FrTable( const FrTable& table );
      FrTable( const std::string& name, INT_4U nrows );

      FrTable( const std::string& Name,
	       const std::string& Comment,
	       nRow_type NRows,
	       std::vector< STRING > ColumnNames );

      FrTable( const FrTableNPS& Source );

      FrTable( Previous::FrTable& Source, Common::IStream* Stream );

      virtual ~FrTable( );

      //-----------------------------------------------------------------
      /// \brief Stream size of object
      ///
      /// \return
      ///     The number of bytes needed to write or read this
      ///     class of object to or from a stream.
      //-----------------------------------------------------------------
      virtual cmn_streamsize_type Bytes( const Common::StreamBase& Stream ) const;

      virtual FrTable* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      const std::string& GetNameSlow( ) const;

      //: Merge with another FrAdcData
      FrTable& Merge( const FrTable& RHS );

      static std::auto_ptr< Object >
      Promote( INT_2U Source, Object* Obj, Common::IStream* Stream );

      //: Concatination operator
      FrTable& operator+=( const FrTable& RHS );

      //-----------------------------------------------------------------
      /// \brief Dictionary class id for this structure.
      //-----------------------------------------------------------------
      static const Common::FrameSpec::Info::frame_object_types CLASS_ID
      = Common::FrameSpec::Info::FSI_FR_TABLE;

      //-----------------------------------------------------------------
      /// \brief Full description for the FrTable structure
      ///
      /// \return
      ///     A pointer to a static Description object containing
      ///     the full description of an FrTable structure for
      ///     the current frame specification.
      //-----------------------------------------------------------------
      static const Common::Description* StructDescription( );

      //-----------------------------------------------------------------
      /// \brief Name of the frame specification structure
      ///
      /// \return
      ///     A pointer to a static address containing the name
      ///     to be associated with this structure as specified
      ///     by the frame specification.
      //-----------------------------------------------------------------
      static const char* const StructName( );

      //-----------------------------------------------------------------
      /// \brief comparison operator
      ///
      /// \return
      ///     True if the two FrTable objects being compared are
      ///     equivelant, false otherwise.
      //-----------------------------------------------------------------
      bool operator==( const FrTable& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief comparison operator
      //-----------------------------------------------------------------
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Up grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrTable( Common::IStream& Stream );
    };

    inline const char* const FrTable::
    StructName( )
    {
      static const char* const name = "FrTable";
      return name;
    }

    inline bool FrTable::
    operator==( const FrTable& RHS ) const
    {
      return ( ( m_data == RHS.m_data )
	       && ( m_refs == RHS.m_refs ) );
    }

    inline std::auto_ptr< Common::FrameSpec::Object > FrTable::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrTable, FrTable >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrTable_HH */

